/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */

#ifndef ARCH_CC1010_RF_H_
#define ARCH_CC1010_RF_H_

#include <arch/cc1010/phal.h>

/* TODO: improve documentation!!! */

#define UDP_HEADER_SIZE 	12

#define RF_HEADER_SIZE		6
#define RF_PAYLOAD_SIZE 	63
#define RF_CRC_SIZE		 	2

// buffer sizes
#define RF_PREAMBLE_LENGTH_RECEIVE	30	// detected bits
#define RF_PREAMBLE_LENGTH_SEND		12 	//bytes

#define RF_SYNC_BYTE		0xA5

// rssi stuff
#define 	RF_RSSI_THRESHOLD					65
#define 	RF_LBT_CHECKS						2
#define 	LBT_GUARANTEED_WAITNG_TIME			10
#define		RSSI_MEASURE_TIMES_MEANS			1		// # means are compared with threshold
#define		RSSI_MEASURE_TIMES_SAMPLES			1		// # samples are taken for one mean
#define		RSSI_MEASURE_TIMES_SHIFT			0		// 2^x is RSSI_MEASURE_TIMES_SAMPLES
#define		RF_LBT_LEAST_TIMEOUT				70		// at least one try to send a packet is possible in this time






extern RF_RXTXPAIR_CALDATA xdata RF_CALDATA;	// Calibration data, wird in cul oder hal angelegt


#define RF_CRC_NOT_DEFINED	0
#define RF_CRC_OK			1
#define RF_CRC_BAD			2

// #define RSSI_THRESHOLD		0x49


extern code RF_RXTXPAIR_SETTINGS RF_SETTINGS315a;
extern code RF_RXTXPAIR_SETTINGS RF_SETTINGS869a;

extern RF_RXTXPAIR_SETTINGS code RF_SETTINGS;
extern RF_RXTXPAIR_CALDATA xdata RF_CALDATA;	// Calibration data

extern uint16_t xdata 	cc_clock_counter;
// receive
extern uint8_t xdata rf_header_receive[ RF_HEADER_SIZE ];
extern uint8_t xdata rf_payload_receive[ RF_PAYLOAD_SIZE ];
extern uint8_t xdata rf_crc_receive[ RF_CRC_SIZE ];
extern uint8_t xdata rf_new_data_receive;
extern uint8_t xdata rf_payload_length_receive;
extern uint8_t xdata rf_receive_buffer_locked;

//send
extern uint8_t xdata rf_header_send[ RF_HEADER_SIZE ];
extern uint8_t xdata rf_payload_send[ RF_PAYLOAD_SIZE ];
extern uint8_t xdata rf_crc_send[ RF_CRC_SIZE ];
extern uint8_t xdata rf_payload_length_send;
extern uint8_t xdata rf_tuple_length_position_send;
extern uint8_t xdata rf_seq_number_send;




/* Prototypes */

void RFInit(code RF_RXTXPAIR_SETTINGS * rf_settings, uint8_t hi_local_osc);

//
uint8_t RFSendPacketPolled( uint16_t timeout );
//uint8_t RFSendPacketWithInterrupts();
void RFReceivePacketPolled() interrupt INUM_RF;
//void RFReceivePacketWithInterrupts() interrupt INUM_RF;
//void RFSendAndReceivePacketWithInterrupts() interrupt INUM_RF;
void RFEnablePreambleDetection();
void RFSetModeTransmit();
void RFSetModeReceive();
void RFSetModeOff();
void RFCalibrate();
void RFCalcCRC16New( uint8_t* crc, uint8_t* header, uint8_t* payload, uint8_t payload_length );
uint8_t RFSendPacket();
void RFWaitForReady();
void RFEnablePreambleDetection();
void RFSetModeTransmit();
void RFSetModeReceive();
void RFSetModeOff();
void RFCalibrate();
void RFInitRandom();
void RFCalcCRC16( uint8_t* crc, uint8_t* header, uint8_t header_size, uint8_t* payload, uint8_t payload_length );
uint8_t RFChannelFree();
void RFRssiAdjust();
uint8_t RFRssiGet();
void RFInitRandom();
void RFNewRandom();

void RFSetByteMode();
void RFSetBitMode();
uint8_t waitsend(uint8_t x);


#endif /*ARCH_CC1010_RF_H_*/
