/*
 * This is the buffer management header file template.
 * Define BUFFER_PREFIX, BUFFER_LINES and BUFFER_SIZE_PER_LINE
 * and then include it.
 * 
 * BUFFER_PREFIX   the prefix that will be expanded before any
 *                 function name or variable name to make it unique
 *                 
 * BUFFER_LINES    number of buffer lines to allocate
 * 
 * BUFFER_SIZE_PER_LINE  number of bytes per buffer line
 */

/* BUFFER_PREFIX, BUFFER_LINES and BUFFER_SIZE_PER_LINE have to be defined! */ 
#ifndef BUFFER_PREFIX
	#error You have to define BUFFER_PREFIX
#endif
#ifndef BUFFER_LINES
	#error You have to define BUFFER_LINES
#endif
#ifndef BUFFER_SIZE_PER_LINE
	#error You have to define BUFFER_SIZE_PER_LINE
#endif

/* 
 * undefine macros in case they have already been defined
 * by another inclusion of this template
 */
#undef ADD_PREFIX
#undef PREFIX_CONCAT1
#undef PREFIX_CONCAT2

/* 
 * concatenation macros
 * need indirection over PREFIX_CONCAT1() so that BUFFER_PREFIX will
 * be expanded correctly
 */
#define ADD_PREFIX(a) PREFIX_CONCAT1(BUFFER_PREFIX,a)
#define PREFIX_CONCAT1(a,b) PREFIX_CONCAT2(a,b)
#define PREFIX_CONCAT2(a,b) a##b

typedef struct ADD_PREFIX(_buffer_management_type_)
{
	uint8_t allocated : 1;
	uint8_t reserved  : 7;
} ADD_PREFIX(_buffer_management_type);

extern uint8_t XRAM ADD_PREFIX(_buffer)[BUFFER_LINES][BUFFER_SIZE_PER_LINE];

uint8_t ADD_PREFIX(Allocate)();
void ADD_PREFIX(Release)( uint8_t line );
