#ifndef UTIL_TIMEOUT_H
#define UTIL_TIMEOUT_H

#include <util/rtc.h>

/* max number of timeouts that can be allocated */
#define MAX_TIMEOUTS 15

/* define timout type */
typedef uint8_t timeout_t;
/* timeout type size in bytes */
#define TIMEOUT_SIZE (sizeof(timeout_t))  

/** @brief Update time out counters
 * 
 * Updates time out counters base on the time that has gone by since last update.
 * Call as often as possible. Needs running real time clock to work.
 */
void TimeoutUpdate( void );

/** @brief Allocate a timeout counter
 * 
 * Retrieves a free timeout counter, when available
 * 
 * @return  pointer to timeout counter
 *          NULL if all timeouts are already allocated     
 */
timeout_t* TimeoutAllocate( void );

#endif /*UTIL_TIMEOUT_H*/
