#ifndef ARCH_CC1010_CC1010_C_
#define ARCH_CC1010_CC1010_C_

#include <arch/cc1010/phal.h>
#include <net/acl.h>

uint16_t xdata 	cc_clock_counter;
uint8_t  xdata 	cc_little_counter;

// to save id in rom and ram
byte code at CC_ROM_ADDRESS cc_my_id_0_rom = 0;
byte code at (CC_ROM_ADDRESS + 1) cc_my_id_1_rom = 0;
byte code at (CC_ROM_ADDRESS + 2) cc_my_id_2_rom = 0;
byte code at (CC_ROM_ADDRESS + 3) cc_my_id_3_rom = 0;
byte code at (CC_ROM_ADDRESS + 4) cc_my_id_check_rom = 0;

uint8_t xdata cc_my_id_read_rom;
uint8_t xdata cc_my_id[ 4 ];
uint8_t xdata at CC_RAM_FOR_ROM_BUFFER_ADDRESS cc_ram_for_rom_buffer[ 128 ];

void phalChipInit(uint8_t param)
{
	(void) param;

	/* TODO: doku */
	P0DIR = P0DIR | 15;	//allin
	P1DIR = 0xFF;		//allin
	P2DIR = 0xFF;		//allin
	P3DIR = P3DIR | 31;	//allin

	/* TODO: doku */
	// watchdog
	WDT = 0x13;
	WDT = 0x13;
	WDT_ENABLE(FALSE);

	// Set optimum settings for speed and low power consumption????
	MEM_NO_WAIT_STATES();
	FLASH_SET_POWER_MODE(FLASH_STANDBY_BETWEEN_READS);

	phalGlobalInterruptDisable();

	phalSPIDisable();

	phalADCInit();
	phalADCEnable();

	phalUart1Init();
}

//**********************************************************************************************

void phalADCDisable()
{
	bit_set( ADCON, 7 );	//power down
}

void phalADCEnable()
{
	bit_clear( ADCON, 7 );	//power up
}

void phalADCInit()
{
	/* TODO: doku */
	bit_set( ADCON, 7 );	//power down
	bit_clear( ADCON, 5 );	//single conversion mode
	bit_clear( ADCON, 4 );	//single conversion mode
	phalADCSetVDDReference();	//set VDD as reference

	phalADCSetChannel( 0 );		//select AD0

	ADCON2 = 4;					// no interrupt on adc, clear adc interrupt flag, timing is 4*16
	phalADCEnable();
}

void phalADCSetVDDReference()
{
	bit_clear( ADCON, 3 );
	phalDelayUs( 5 );
}

void phalADCSetInternalReference()
{
	bit_set( ADCON, 3 );
	phalDelayUs( 100 );
}

void phalADCSetChannel(uint8_t channel)
{
	channel=channel&0x3;	//mask only two lowest bit
	if(( ADCON & 3 ) != channel )
	{
		ADCON=ADCON&0xFC;		//clear lowest two bit
		ADCON=ADCON|channel;	//set lowest two bit to channel

		phalDelayUs( 50 );
	}
}

/**
* Reads a sampled value from the A/D converter. Only one byte.
* @return sampled value as uint8_t
*/
uint8_t phalADCRead8()
{
	bit_set( ADCON, 2 );		// set run
	while( bit_test( ADCON, 2) ) ;		//wait for ready
	return (ADDATH * 64) | (ADDATL / 4) ;
}

/**
* Reads a sampled value from the A/D converter. Full 10bit of the ADC
* @return sampled value as uint16_t, where the bit 3-7 are set to 0.
*/
uint16_t phalADCRead10()
{
	bit_set( ADCON, 2 );		// set run
	while( bit_test( ADCON, 2 ) );		//wait for ready

	return  ( ((uint16_t) ADDATH) << 8 ) + (uint16_t) (ADDATL);
}

//**********************************************************************************************

/* TODO: make option an enum or define with meaningful name */
void phalTimer0Init(uint8_t option)
{
	// at the moment only one option: 16 bit timer, sourced by the system clock=quartz freq div by 12,
	// the  16 bit will overflow after 1/14.mhz*12*2^6=53ms, one Timer 0 high step is 208 us

	if(option==1)
	{
		TR0=0;	// timer 0 stopped
		TR0=0;	// timer 0 stopped
		TR0=0;	// timer 0 stopped

		bit_clear(TMOD,3);	// TMR0 no external gate

		bit_clear(TMOD,2);	// TMR0 internal clock

		bit_clear(TMOD,1);	//   }
		bit_set(TMOD,0);	//	 }  set timer 0 as 16 bit timer
		bit_clear(CKCON,3);	// source of TMR0 is system clock divided by 12

		TF0=0;	// timer0  interrupt flag off	= off
		ET0=1;	// timer0 interrupt enable flag	 = disable, don't generate interrupts

		TR0=1;	//start timer

	}
	if(option==2)
	{
		bit_clear(TMOD,3);	// GATE0: Timer 0 will clock only when tcon.tr1 is set (the run bit)
		bit_clear(TMOD,2);	//C0: Timer 0 is clocked by system div 4 or div 12
		bit_set(TMOD,1);	// M0.1:
		bit_clear(TMOD,0);	// M0.0:  mode is 2 (0b10)= 8-bit counter with auto-reload

		bit_set(TCON,4);	// enable timer 0

		bit_set(CKCON,3);	// timer 0 uses sys clock div 4
		TH0=212;			// the preload value;	// results in 42 kHz

		bit_clear(IE,1);	// disable TIMER0 interrupt
	}
}

void phalTimer0Start()
{
	bit_set(TCON,4);	// enable timer 0
}

void phalTimer0Stop()
{
	bit_set(TCON,4);	// disable timer 0
}

uint8_t phalTimer0Overflow()
{
	return bit_test( TCON, 5 );
}

void phalTimer0ClearOverflow()
{
	bit_clear( TCON, 5 );
}

uint16_t phalTimer0Get()
{
	return ( ((uint16_t) TH0) * 256) + (uint16_t) TL0;
}

void phalTimer0Set(uint16_t value)
{
	TH0 = value >> 8;
	TL0 = value;
}

void phalTimer2Init()
{
		bit_clear(TCON2,0);	// timer mode

//		T2PRE = 225;		// reload value 57826 in order to get an overflow every second.
//		T2 = 226;			// every second is 8mue too short, that is, after 34 h 1 s less.
		T2PRE = 14;			// reload value 3613 in order to get an overflow every 1/16 second.
		T2 = 29;

//		bit_clear( EIE, ET2 );	// disable TIMER2 interrupt

		EIE &= 0xFD;	//BIN 11111101
		phalTimer2ClearOverflow();	// clear overflow flag

		phalTimer2Start();
}

void phalTimer2Start()
{
	bit_set(TCON2, 1);	// enable timer 0
}

void phalTimer2Stop()
{
	bit_set(TCON2, 1);	// disable timer 0
}

uint8_t phalTimer2Overflow()
{
	return bit_test( EXIF, 5 );
}

void phalTimer2ClearOverflow()
{
	bit_clear( EXIF, 5 );
}

void phalTimer3Init(byte option)
{
	//PWM option
	if (option==1)
	{
		bit_clear(TCON2,3);	//disable timer 3
		bit_set(TCON2,2);	//turn timer 3 to pwm mode
		T3PRE=230;			// set the prescaler for the timer 3 speed ca 250hz
		T3=127;				// duty cyle is 50%;
		bit_set(TCON2,3);	// run timer 3
	}

}

//**********************************************************************************************

void phalUart0Init( uint8_t mode )
{
	if( mode == 1 )
	{
        /* Setup pins for UART0 */
        PORTDIRBIT(3, 0, PIN);
        PORTDIRBIT(3, 1, POUT);
        PORTBIT(3, 1) = 1;

        /* Setup baudrate & timer 1 */
        CKCON&=239;                /* T1M=1 */
        PCON|=0x80;                 /* SMOD0=1 */
        TH1= 224;   //256 - ((((ulong)(clkFreq)*10)/((ulong)(baudRate)/100*32) +1)/2);
        TR1=1;
        TMOD=(TMOD&0x0F)|0x20;
        halTimer1Usage++;

        /* Setup SCON0 & interrupts */
        SCON0=64;
	}
}

void phalUart0ReceiveOn()
{
	bit_set( SCON0, 4 );
}

void phalUart0ReceiveOff()
{
	bit_clear( SCON0, 4 );
}

void phalUart0SendByte( uint8_t datum )
{
	TI_0 = 0;
	UART0_SEND( datum );
}

uint8_t phalUart0ByteSent()
{
	return TI_0;
}

uint8_t phalUart0ReceiveBytePolled()
{
	uint8_t datum;

		UART0_WAIT_AND_RECEIVE( datum );
		return datum;
}

uint8_t phalUart0CheckForNewData()
{
	if( RI_0 ) return TRUE;
	else return FALSE;
}

void phalUart1Init()
{
	UART1_SETUP(115200, 14746, UART_NO_PARITY | UART_RX_TX | UART_POLLED);
}

void phalUart1ReceiveOn()
{
	bit_set( SCON1, 4 );
}

void phalUart1ReceiveOff()
{
	bit_clear( SCON1, 4 );
}

void phalUart1SendByte( uint8_t datum )
{
	TI_1 = 0;
	UART1_SEND( datum );
}

uint8_t phalUart1ByteSent()
{
	return TI_1;
}

void phalUart1SendBytePolledEscaped( uint8_t datum )
{
	if(( datum == 0 ) || ( datum == 192 ) || ( datum == 219 ))
	{
		UART1_WAIT_AND_SEND(219);
		datum++;
	}
	UART1_WAIT_AND_SEND( datum );
}

uint8_t phalUart1ReceiveBytePolled()
{
	uint8_t datum;

		UART1_WAIT_AND_RECEIVE( datum );

		return datum;
}

uint8_t phalUart1CheckForNewData()
{
	if( RI_1 ) return TRUE;
	else return FALSE;
}

void phalUart1ClearReceiveFlag()
{
	RI_1 = 0;
}

//**********************************************************************************************

void phalSPIDisable()
{
	bit_clear( SPCR, 5 );
}

//****************************************************************************************************************************

void phalClockInit()
{
	phalTimer2Init();
	cc_clock_counter = 0;
	cc_little_counter = 0;
}

void phalUpdateClock()
{
	if( phalTimer2Overflow())
	{
		++cc_little_counter;
		if( cc_little_counter >= 16 )
		{
			++cc_clock_counter;
			cc_little_counter -= 16;
		}
		phalTimer2ClearOverflow();
	}
}

uint16_t phalClockGetInterval( uint16_t start_time )
{
	uint16_t dummy;

		dummy = cc_clock_counter;

		if( dummy >= start_time ) return dummy - start_time;
		else return (65536 - start_time) + dummy;
}

uint8_t phalClockGetLittleInterval( uint8_t start_little_time )
{
	uint8_t dummy;

		dummy = cc_little_counter;

		if( dummy >= start_little_time ) return dummy - start_little_time;
		else return 16 - start_little_time + dummy;
}

//**********************************************************************************************

uint8_t phalGetID()
{
	// check if id had been written
	_asm
	mov dptr, #CC_ROM_ADDRESS + #CC_ROM_MY_ID_CHECK_POSITION
	clr a
	movc a,@a+dptr

	mov dptr, #_cc_my_id_read_rom
	movx @dptr, a
	_endasm;

	if( cc_my_id_read_rom == CC_MY_ID_CHECK )
	{
		// get first id byte
		_asm
		mov dptr, #_cc_my_id_0_rom
		clr a
		movc a,@a+dptr

		mov dptr, #_cc_my_id
		movx @dptr, a
		_endasm;

//		cc_my_id[ 0 ] = cc_my_id_read_rom;

		// get second id byte
		_asm
		mov dptr, #_cc_my_id_1_rom
		clr a
		movc a,@a+dptr

		mov dptr, #_cc_my_id + 0x01
		movx @dptr, a
		_endasm;

//		cc_my_id[ 1 ] = cc_my_id_read_rom;

		// get third id byte
		_asm
		mov dptr, #_cc_my_id_2_rom
		clr a
		movc a,@a+dptr

		mov dptr, #_cc_my_id + 0x02
		movx @dptr, a
		_endasm;

//		cc_my_id[ 2 ] = cc_my_id_read_rom;

		// get fourth id byte
		_asm
		mov dptr, #_cc_my_id_3_rom
		clr a
		movc a,@a+dptr

		mov dptr, #_cc_my_id + 0x03
		movx @dptr, a
		_endasm;

//		cc_my_id[ 3 ] = cc_my_id_read_rom;

		return 1;
	}
	else
	{
		cc_my_id[ 0 ] = CC_DEFAULT_ID_0;
		cc_my_id[ 1 ] = CC_DEFAULT_ID_1;
		cc_my_id[ 2 ] = CC_DEFAULT_ID_2;
		cc_my_id[ 3 ] = CC_DEFAULT_ID_3;

		return 0;
	}
}

void phalWriteID( uint8_t* id )
{
	uint8_t formerEA;
	uint8_t i;

		for( i = 0; i < 4; ++i )
		{
			cc_ram_for_rom_buffer[ CC_ROM_MY_ID_POSITION + i ] = *( id + i );
		}

		cc_ram_for_rom_buffer[ CC_ROM_MY_ID_CHECK_POSITION ] = CC_MY_ID_CHECK;

		formerEA = EA;
		EA = 0;
		EICON |= 0x20;

		FLTIM = ( byte )( BOARD_CLK_FREQ >> 8 ) / 3;
		FLCON = ( CC_RAM_FOR_ROM_BUFFER_ADDRESS >> 7 ) | 16;
		FLADR = CC_ROM_ADDRESS >> 7;

		PCON |= 1;

		EA = formerEA;

		phalGetID();
}

void phalEraseID()
{
	uint8_t formerEA;
	uint8_t i;

		for( i = 0; i < 4; ++i )
		{
			cc_ram_for_rom_buffer[ CC_ROM_MY_ID_POSITION + i ] = 0;
		}

		cc_ram_for_rom_buffer[ CC_ROM_MY_ID_CHECK_POSITION ] = 0;

		formerEA = EA;
		EA = 0;
		EICON |= 0x20;

		FLTIM = ( byte )( BOARD_CLK_FREQ >> 8 ) / 3;
		FLCON = ( CC_RAM_FOR_ROM_BUFFER_ADDRESS >> 7 ) | 16;
		FLADR = CC_ROM_ADDRESS >> 7;

		PCON |= 1;

		EA = formerEA;

		cc_my_id[ 0 ] = CC_DEFAULT_ID_0;
		cc_my_id[ 1 ] = CC_DEFAULT_ID_1;
		cc_my_id[ 2 ] = CC_DEFAULT_ID_2;
		cc_my_id[ 3 ] = CC_DEFAULT_ID_3;
}

void phalCheckID()
{
	uint8_t i;
	uint8_t id[ 4 ];

		i = 25;
		if( !phalGetID())
		{
			SET_LED( RED, LED_ON );
			while( i != 0 )
			{
				if( ACLDataIsNew() )
				{
					if( ACLFoundReceivedType( ACL_TYPE_CCI_HI, ACL_TYPE_CCI_LO ))
					{
						id[ 0 ] = *ACLGetReceivedData( ACL_TYPE_CCI_HI, ACL_TYPE_CCI_LO );
						id[ 1 ] = *( ACLGetReceivedData( ACL_TYPE_CCI_HI, ACL_TYPE_CCI_LO ) + 1 );
						id[ 2 ] = *( ACLGetReceivedData( ACL_TYPE_CCI_HI, ACL_TYPE_CCI_LO ) + 2 );
						id[ 3 ] = *( ACLGetReceivedData( ACL_TYPE_CCI_HI, ACL_TYPE_CCI_LO ) + 3 );
						phalWriteID( id );
						rf_header_send[ 2 ] = cc_my_id[ 0 ];
						rf_header_send[ 3 ] = cc_my_id[ 1 ];
						rf_header_send[ 4 ] = cc_my_id[ 2 ];
						rf_header_send[ 5 ] = cc_my_id[ 3 ];
						SET_LED( RED, LED_OFF );
						break;
					}
					ACLSetDataToOld();
				}
				phalDelayMs( 200 );
				--i;
			}
			SET_LED( RED, LED_OFF );
		}
}

#endif /*ARCH_CC1010_CC1010_C_*/
