#ifndef ARCH_CC1010_PHALCRITICALSECTION_C_
#define ARCH_CC1010_PHALCRITICALSECTION_C_

#include <arch/cc1010/phal.h>

/* this implementation is quite long, thus not very fast...
 * a shorter version would be nice, since the time spent in a CS should be as short as possible
 *
 * I can think of some other ways to do it, but it would require macros and assembler :-) --Martin
 */

/* abs( nestedCriticalSections ) == level of nesting
 *
 * nestedCriticalSections > 0: interrupts were enabled prior to entering the critical section(s)
 * nestedCriticalSections < 0: interrupts were disabled prior to entering the critical section(s)
 */
int8_t nestedCriticalSections = 0;

/* does not account for too many levels of nesting ( > 127 ) -> no overflow handling */
void phalCriticalSectionEnter()
{
	if( nestedCriticalSections == 0 )
	{
		/* increment or decrement depending on whether interrupts are enabled or not */
		if( EA == 0 ) --nestedCriticalSections;
		else
		{
			++nestedCriticalSections;
			phalGlobalInterruptDisable();
		}

		return;
	}
	else if( nestedCriticalSections < 0 )
	{
		--nestedCriticalSections;
	}
	else /* nestedCriticalSections > 0 */
	{
		++nestedCriticalSections;
	}	
}

void phalCriticalSectionExit()
{
	if( nestedCriticalSections > 0 )
	{
		if( --nestedCriticalSections == 0 )
			phalGlobalInterruptEnable();
	}
	else if( nestedCriticalSections < 0 )
	{
		++nestedCriticalSections;
	}
}

#endif /*ARCH_CC1010_PHALCRITICALSECTION_C_*/

