/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */


#ifndef ARCH_CC1010_RF_C_
#define ARCH_CC1010_RF_C_

#include <arch/cc1010/rf.h>
#include INC_BOARD( board.h )

code RF_RXTXPAIR_SETTINGS * my_rf_settings;

RF_RXTXPAIR_CALDATA xdata RF_CALDATA;	// Calibration data

// receive
uint8_t xdata rf_header_receive[ RF_HEADER_SIZE ];
uint8_t xdata rf_payload_receive[ RF_PAYLOAD_SIZE ];
uint8_t xdata rf_crc_receive[ RF_CRC_SIZE ];
uint8_t xdata rf_new_data_receive;
uint8_t xdata rf_payload_length_receive;
uint8_t xdata rf_receive_buffer_locked;

//send
uint8_t xdata rf_header_send[ RF_HEADER_SIZE ];
uint8_t xdata rf_payload_send[ RF_PAYLOAD_SIZE ];
uint8_t xdata rf_crc_send[ RF_CRC_SIZE ];
uint8_t xdata rf_payload_length_send;
uint8_t xdata rf_tuple_length_position_send;
uint8_t xdata rf_seq_number_send;

//crc
uint8_t xdata rf_crc_status;

//rssi
uint8_t	xdata rf_rssi_threshold;
uint8_t 	xdata 	rf_lbt_checks;

//random
uint8_t  	xdata 	rf_random_l;
uint8_t  	xdata 	rf_random_h;


	// X-tal frequency: 14.745600 MHz
	// RF frequency A: 314.611846 MHz	Rx
	// RF frequency B: 314.611846 MHz	Tx
	// RX Mode: High side LO
	// Frequency separation: 20 kHz
	// Data rate: 19.2 kBaud
	// Data Format: NRZ
	// RF output power: 0 dBm
	// IF/RSSI: RSSI Enabled
RF_RXTXPAIR_SETTINGS code RF_SETTINGS315a = {
		0xA3, 0x2F, 0x0B,    // Modem 0, 1 and 2
		0x45, 0x40, 0x00,    // Freq A
		0x45, 0x35, 0x79,    // Freq B	//BB
		0x04, 0x20,          // FSEP 1 and 0
		0x68,                // PLL_RX
		0x68,                // PLL_TX
		0x44,                // CURRENT_RX
		0x81,                // CURRENT_TX
		0x0A,                // FREND
		0x50,                // PA_POW	// 0x70
		0xC0,                // MATCH
		0x00,                // PRESCALER
		};

	// X-tal frequency: 14.745600 MHz
	// RF frequency A: 868.631600 MHz	Rx
	// RF frequency B: 868.631600 MHz	Tx
	// RX Mode: High side LO
	// Frequency separation: 64 kHz
	// Data rate: 19.2 kBaud
	// Data Format: NRZ
	// RF output power: 0 dBm
	// IF/RSSI: RSSI Enabled
RF_RXTXPAIR_SETTINGS code RF_SETTINGS869a = {
		0xA3, 0x2F, 0x13,    // Modem 0, 1 and 2
		0x58, 0x60, 0x00,    // Freq A
		0x58, 0x5B, 0xC8,    // Freq B
		0x01, 0x8F,          // FSEP 1 and 0
		0x30,                // PLL_RX
		0x30,                // PLL_TX
		0x6C,                // CURRENT_RX
		0xF3,                // CURRENT_TX
		0x32,                // FREND
		0xA0,                // PA_POW
		0x00,                // MATCH
		0x00,                // PRESCALER
		};

/*****************************************************************************************************************/
//RF functions.
/*****************************************************************************************************************/
void RFInit(RF_RXTXPAIR_SETTINGS code * rf_settings, uint8_t hi_local_osc)
{
	(void) hi_local_osc; //dummy code

	my_rf_settings = rf_settings;

	halRFCalib(my_rf_settings, &RF_CALDATA);

//	halRFCalib(&RF_SETTINGS, &RF_CALDATA);

    INT_SETFLAG (INUM_RF, INT_CLR);
    INT_PRIORITY (INUM_RF, INT_HIGH);
    INT_ENABLE (INUM_RF, INT_ON);

    RF_SET_BYTEMODE();
    RF_SET_PREAMBLE_COUNT( RF_PREAMBLE_LENGTH_RECEIVE );

    rf_receive_buffer_locked = FALSE;

	//XXX missing: the treatment of hi_local or lo_local oscillator;
	// influences the inversion of received bytes, syncbytes etc; now: only hi_lo is allowed at the moment


	RF_SET_SYNC_BYTE(~RF_SYNC_BYTE);

 	RFSetModeReceive();

 	RFInitRandom();

	rf_rssi_threshold = RF_RSSI_THRESHOLD;
	rf_lbt_checks = 0;

 	rf_crc_status = RF_CRC_NOT_DEFINED;
}


uint8_t RFSendPacketPolled( uint16_t timeout )
{
	uint8_t 	i;
	uint8_t 	crc[ RF_CRC_SIZE ];
	uint16_t 	time;
	uint8_t 	waiting_time;


		SET_LED( RED, LED_ON );

		time = 0;
		if( timeout < RF_LBT_LEAST_TIMEOUT ) timeout = RF_LBT_LEAST_TIMEOUT;

		while( 1 )
		{
			if( RFChannelFree())
			{
				if( rf_seq_number_send++ == 64 ) rf_seq_number_send = 0;

				rf_header_send[ 1 ] = rf_seq_number_send;

				RFCalcCRC16( crc, rf_header_send, RF_HEADER_SIZE, rf_payload_send, rf_payload_length_send );

				phalGlobalInterruptDisable();

				RFSetModeTransmit();
				RF_SEND_BYTE(0xAA);
				RF_START_TX();

				// send preamble
				for( i = 0; i < RF_PREAMBLE_LENGTH_SEND - 1; ++i )
				{
					RF_WAIT_AND_SEND_BYTE(0xAA);
				}

				RF_WAIT_AND_SEND_BYTE(RF_SYNC_BYTE);

				// send header
				for( i = 0; i < RF_HEADER_SIZE; ++i )
				{
					RF_WAIT_AND_SEND_BYTE( rf_header_send[ i ]);
				}

				// send payloads
				for( i = 0; i < rf_payload_length_send; ++i )
				{
					RF_WAIT_AND_SEND_BYTE( rf_payload_send[ i ]);
				}

				// send crc
				RF_WAIT_AND_SEND_BYTE( crc[ 0 ] );
				RF_WAIT_AND_SEND_BYTE( crc[ 1 ] );

				RF_WAIT_AND_SEND_BYTE(0);			//wichtig, sonst geht crc kaputt!
				RF_WAIT_AND_SEND_BYTE(0);			//wichtig, sonst geht crc kaputt!

				RFSetModeReceive();

				phalGlobalInterruptEnable();

				SET_LED( RED, LED_OFF );

				// now rf_payload_send is free again
				rf_payload_length_send = 0;
				rf_tuple_length_position_send = 0;
				return 1;
			}
			else
			{
				waiting_time = rf_random_l & 63;
				time += (uint8_t)waiting_time;

				if( time >= timeout )
				{
					rf_payload_length_send = 0;
					rf_tuple_length_position_send = 0;
					return 0;
				}

				phalDelayMs( waiting_time + LBT_GUARANTEED_WAITNG_TIME );
			}
		}
}

void RFReceivePacketPolled() interrupt INUM_RF
{
	uint8_t sync_byte;
	uint8_t i;
	uint8_t crc[ RF_CRC_SIZE ];
	uint8_t crc_result = 0;

	rf_crc_status = RF_CRC_NOT_DEFINED;
	if( !rf_receive_buffer_locked )
	{
		INT_SETFLAG(INUM_RF, INT_CLR);
		RF_LOCK_AVERAGE_FILTER(TRUE);

		sync_byte = ~RFBUF;

		if( sync_byte == RF_SYNC_BYTE )
		{
			SET_LED( BLUE, LED_ON );
			for( i = 0; i < RF_HEADER_SIZE; i++ )
			{
				while(!INT_GETFLAG(INUM_RF));
				rf_header_receive[ i ] = ~RFBUF;
				INT_SETFLAG(INUM_RF, INT_CLR);
			}

			rf_payload_length_receive = rf_header_receive[ 0 ] & 0x3f; /* 0x3f == 0b00111111 */

			for( i = 0; i < rf_payload_length_receive; i++ )
			{
				while(!INT_GETFLAG(INUM_RF));
				rf_payload_receive[ i ] = ~RFBUF;
				INT_SETFLAG(INUM_RF, INT_CLR);
			}
			for( i = 0; i < RF_CRC_SIZE; i++ )
			{
				while(!INT_GETFLAG(INUM_RF));
				rf_crc_receive[ i ] = ~RFBUF;
				INT_SETFLAG(INUM_RF, INT_CLR);
			}
			RFEnablePreambleDetection();
			RF_LOCK_AVERAGE_FILTER(FALSE);

			RFCalcCRC16( crc, rf_header_receive, RF_HEADER_SIZE, rf_payload_receive, rf_payload_length_receive );

			for( i = 0; i < RF_CRC_SIZE; ++i ) if( crc[ i ] == rf_crc_receive[ i ]) ++crc_result;

			if( crc_result == RF_CRC_SIZE )
			{
				rf_crc_status = RF_CRC_OK;
				rf_new_data_receive = 1;
				SET_LED( BLUE, LED_OFF );
			}
			else
			{
				rf_crc_status = RF_CRC_BAD;
				rf_new_data_receive = 0;
			}
		}
	}
	INT_SETFLAG(INUM_RF, INT_CLR);
}

void RFEnablePreambleDetection()
{
	PDET &= ~0x80;
	PDET |= 0x80;
}

void RFSetModeTransmit()
{
	//halRFCalib(my_rf_settings, &RF_CALDATA);
	halRFSetRxTxOff(RF_TX, my_rf_settings, &RF_CALDATA);
}

void RFSetModeReceive()
{
	RFEnablePreambleDetection();
	halRFSetRxTxOff(RF_RX, my_rf_settings, &RF_CALDATA);
	RF_START_RX();
}

void RFSetModeOff()
{
	halRFSetRxTxOff(RF_OFF, my_rf_settings, &RF_CALDATA);
}

void RFCalibrate()
{
	halRFCalib(my_rf_settings, &RF_CALDATA);
}

#pragma save		// important because function can be called in interrupt and overwrites main ram.
#pragma nooverlay
void RFCalcCRC16( uint8_t* crc, uint8_t* header, uint8_t header_size, uint8_t* payload, uint8_t payload_length )
{
	uint8_t i, tmp;

	uint8_t crc_lo;
	uint8_t crc_hi;

	crc_lo = 0;
	crc_hi = 0;

	for( i = 0; i < header_size; ++i )
	{
		tmp = crc_hi;
		crc_hi = crc_lo;
		crc_lo = tmp;
		crc_hi ^= header[i];
		crc_hi ^= crc_hi >> 4;
		crc_lo ^= crc_hi << 4;
		crc_lo ^= crc_hi >> 3;
		crc_hi ^= (crc_hi << 4) << 1;
	}

	for( i = 0; i < payload_length; ++i )
	{
		tmp = crc_hi;
		crc_hi = crc_lo;
		crc_lo = tmp;
		crc_hi ^= payload[i];
		crc_hi ^= crc_hi >> 4;
		crc_lo ^= crc_hi << 4;
		crc_lo ^= crc_hi >> 3;
		crc_hi ^= (crc_hi << 4) << 1;
	}

	*crc = crc_lo;
	*( crc + 1 ) = crc_hi;
}
#pragma restore

//****************************************************************************************************************************

uint8_t RFChannelFree()
{
	uint8_t counter = 0;
	uint8_t j;
	uint8_t times;

		RFNewRandom();
		times = RSSI_MEASURE_TIMES_MEANS;
		for( j = 0; j < times; j++ )
		{
			if( RFRssiGet() >= rf_rssi_threshold )
			{
				++counter;
			}
		}
		if( counter == times )
		{
			++rf_lbt_checks;
			if( rf_lbt_checks == RF_LBT_CHECKS )
			{
				rf_lbt_checks = 0;
				return 1;
			}
		}
		else
		{
			rf_lbt_checks = 0;
//			SET_LED( BLUE, LED_OFF );
//			SET_LED( RED, LED_ON );
		}
		return 0;
}

uint8_t RFRssiGet()
{
	uint16_t result = 0;
	uint8_t i;

		phalADCEnable();
		phalADCSetChannel( BOARD_ADC_CHANNEL_RSSI );

		for( i = 0; i < RSSI_MEASURE_TIMES_SAMPLES; ++i )
		{
			result += phalADCRead8();
		}

		result = result >> RSSI_MEASURE_TIMES_SHIFT;

		return result;
}

//****************************************************************************************************************************

void RFInitRandom()
{
	rf_random_h = CC_DEFAULT_ID_0;
}

void RFNewRandom()
{
	uint8_t dummy1;
	uint8_t dummy2;

		// first the 16bit randomfunction
		dummy1=0;dummy2=0;
		if (bit_test(rf_random_l,3)) dummy1=1;
		if (bit_test(rf_random_h,4)) dummy1^=1;
		if (bit_test(rf_random_h,6)) dummy2=1;
		if (bit_test(rf_random_h,7)) dummy2^=1;
		dummy1=dummy1^dummy2;

		// this makes a 16 bit shift left with the random_l& h registers (they might not be next to each other)
		dummy2=0;
		if (bit_test(rf_random_l,7)) dummy2=1;	// save carry bit
		rf_random_l<<=1;							//shift one left
		rf_random_h<<=1;							//shift one left
		if (dummy2==1) bit_set(rf_random_h,0);	// set carry bit
		// this was 16 bit shift
		if(dummy1) bit_set(rf_random_l,0);
}

#endif /*ARCH_CC1010_RF_C_*/

