#ifndef NET_ACL_C_
#define NET_ACL_C_

#include <net/acl.h>

extern uint8_t xdata 	cc_my_id[4];

// functions
/*****************************************************************************************************************/
//ACL functions.
/*****************************************************************************************************************/
uint8_t ACLInit()
{
	//RFInit();

	rf_header_send[ 0 ] = 0;
	rf_header_send[ 1 ] = 1;

	/* my ID */
	if( phalGetID())
	{
		rf_header_send[ 2 ] = cc_my_id[ 0 ];
		rf_header_send[ 3 ] = cc_my_id[ 1 ];
		rf_header_send[ 4 ] = cc_my_id[ 2 ];
		rf_header_send[ 5 ] = cc_my_id[ 3 ];
	}
	else
	{
		rf_header_send[ 2 ] = CC_DEFAULT_ID_0;
		rf_header_send[ 3 ] = CC_DEFAULT_ID_1;
		rf_header_send[ 4 ] = CC_DEFAULT_ID_2;
		rf_header_send[ 5 ] = CC_DEFAULT_ID_3;
	}

	rf_payload_length_send = 0;
	rf_tuple_length_position_send = 0;
	rf_seq_number_send = 0;

	ACLSetDataToOld();

	return 1;
}

// returns1 if packet was started
// returns 0 if still sending another or connection is off..
uint8_t ACLSendPacket( uint16_t timeout )
{
	// insert payload length into header
	rf_header_send[ 0 ] = rf_payload_length_send & 63;

	if( RFSendPacketPolled( timeout )) return 1;
	else return 0;
}

// return 0 if ok
// returns 1 if full
// returns 2 if locked
uint8_t ACLAddNewType( uint8_t type_h, uint8_t type_l)
{
//	if (ACL_send_buffer_locked==true) return 2;						// dont set if reservation is active
	if (( RF_PAYLOAD_SIZE - rf_payload_length_send ) < 3 )
	{
		return 1;													// not enough space
	}
	else
	{
		//place the new type
		rf_payload_send[ rf_payload_length_send ] = type_h;
		rf_payload_send[ rf_payload_length_send + 1 ] = type_l;
		rf_payload_send[ rf_payload_length_send + 2 ] = 0;				//data length is zero
		rf_tuple_length_position_send = rf_payload_length_send + 2;
		rf_payload_length_send += 3;

		return 0;
	}
}

uint8_t ACLAddData( uint8_t datum )
{
//	if (ACL_send_buffer_locked==true) return 2;						// dont set if reservation is active
	if (( RF_PAYLOAD_SIZE - rf_payload_length_send ) < 1 )
	{
		return 1;													// not enough space
	}
	else
	{
		rf_payload_send[ rf_tuple_length_position_send ]++;					//count the acl tuple len up
		rf_payload_send[ rf_payload_length_send ] = datum;					//place the data
		rf_payload_length_send += 1;
		return 0;
	}
}

uint8_t ACLGetRemainingPayloadSpace()
{
	return RF_PAYLOAD_SIZE - rf_payload_length_send;
}

uint8_t ACLGetReceivedPayloadLength()
{
	return rf_payload_length_receive;
}

//looks for the length of context data of the given type, return2 -1 if type wasn't found
int8_t ACLGetReceivedDataLength( uint8_t type_h, uint8_t type_l)
{
	uint8_t i;

		i = 0;

		while( i < rf_payload_length_receive )
		{
			if(( rf_payload_receive[ i ] == type_h ) && ( rf_payload_receive[ i + 1 ] == type_l ))
			{
				return( rf_payload_receive[ i + 2 ]);											//return valid length
			}
			else i += ( rf_payload_receive[ i + 2 ] + 3 );
		}
		return -1;																			// type not found
}

uint8_t ACLFoundReceivedType( uint8_t type_h, uint8_t type_l)
{
	if( ACLGetReceivedDataLength( type_h, type_l ) != -1 ) return 1;	//found!
	else return 0;		//not found
}

uint8_t* ACLGetReceivedData( uint8_t type_h, uint8_t type_l)
{
	uint8_t i;

		i = 0;

		while( i < rf_payload_length_receive )
		{
			if(( rf_payload_receive[ i ] == type_h ) && ( rf_payload_receive[ i + 1 ] == type_l ))
			{
				if( rf_payload_receive[ i + 2 ]) return &rf_payload_receive[ i + 3 ];											//return valid length
			}
			else i += ( rf_payload_receive[ i + 2 ] + 3 );
		}

		return NULL;
	//attention!! if type wasn't found: return is undetermined!!
}

uint8_t ACLGetReceivedByte( uint8_t type_h, uint8_t type_l, uint8_t position)
{
	uint8_t i;

		i = 0;

		while( i < rf_payload_length_receive )
		{
			if(( rf_payload_receive[ i ] == type_h ) && ( rf_payload_receive[ i + 1 ] == type_l ))
			{
				if( rf_payload_receive[ i + 2 ]) return rf_payload_receive[ i + 3 + position ];											//return valid length
			}
			else i += ( rf_payload_receive[ i + 2 ] + 3 );
		}

		return 0;
	//attention!! if type wasn't found: return is undetermined!!
}

void ACLLockReceiveBuffer()
{
	rf_receive_buffer_locked = TRUE;
}

uint8_t ACLReceiveBufferLocked()
{
	return rf_receive_buffer_locked;
}

void ACLReleaseReceiveBuffer()
{
	rf_receive_buffer_locked = FALSE;
}

void ACLSetDataToOld()
{
	rf_new_data_receive = 0;
}

void ACLSetDataToNew()
{
	rf_new_data_receive = 1;
}

uint8_t ACLDataIsNew()
{
	return rf_new_data_receive;
}

#endif /*NET_ACL_C_*/
