#ifndef UTIL_DEBUG_C_
#define UTIL_DEBUG_C_

#include <util/debug.h>
#include <net/acl.h>



// functions
/*****************************************************************************************************************/
//Debug functions.
/*****************************************************************************************************************/
uint8_t DebugInit()
{
	debug_seq_nr = 0;
	debug_payload_length = 0;
	debug_tuple_length_position = 0;

	return 1;
}


// returns1 if packet was started
// returns 0 if still sending another or connection is off..
uint8_t DebugViaUsbridgeSendPacket()
{
	uint8_t crc[ DEBUG_CRC_SIZE ];
	uint8_t i;

		phalGlobalInterruptDisable();

		SET_LED( BLUE, LED_ON );

		if( ++debug_seq_nr == 64 ) debug_seq_nr = 0;

		debug_header_send[ 0 ] 	= 0;										//version
		debug_header_send[ 1 ] 	= debug_payload_length;
		debug_header_send[ 2 ] 	= 0;				//fieldstrength
		debug_header_send[ 3 ] 	= DEBUG_ID_0;									//id0
		debug_header_send[ 4 ] 	= DEBUG_ID_1;									//id1
		debug_header_send[ 5 ] 	= DEBUG_ID_2; 								//id2
		debug_header_send[ 6 ] 	= DEBUG_ID_3; 								//id3
		debug_header_send[ 7 ] 	= rf_header_send[ 2 ];					//id4
		debug_header_send[ 8 ] 	= rf_header_send[ 3 ];					//id5
		debug_header_send[ 9 ] 	= rf_header_send[ 4 ];					//id6
		debug_header_send[ 10 ] = rf_header_send[ 5 ];					//id7
		debug_header_send[ 11 ] = debug_seq_nr & 63;					//sequence number

		RFCalcCRC16( crc, debug_header_send, DEBUG_HEADER_SIZE, rf_payload_receive, debug_payload_length );

		UART1_WAIT_AND_SEND(0);

		// header
		for( i = 0; i < DEBUG_HEADER_SIZE; ++i ) phalUart1SendBytePolledEscaped( debug_header_send[ i ]);

		// payload
		for( i = 0; i < debug_payload_length; ++i ) phalUart1SendBytePolledEscaped( rf_payload_receive[ i ]);

		// crc
		for( i = 0; i < DEBUG_CRC_SIZE; ++i ) phalUart1SendBytePolledEscaped( crc[ i ]);

		UART1_WAIT_AND_SEND(192);
		ACLSetDataToOld();
		SET_LED( BLUE, LED_OFF );

		phalGlobalInterruptEnable();

		debug_payload_length = 0;
		debug_tuple_length_position = 0;
		ACLReleaseReceiveBuffer();
		return 1;
}


// return 0 if ok
// returns 1 if full
// returns 2 if locked
uint8_t DebugViaUsbridgeAddNewType( uint8_t type_h, uint8_t type_l)
{
	ACLLockReceiveBuffer();
	if (( RF_PAYLOAD_SIZE - debug_payload_length ) < 3 )
	{
		return 1;													// not enough space
	}
	else
	{
		//place the new type
		rf_payload_receive[ debug_payload_length ] = type_h;
		rf_payload_receive[ debug_payload_length + 1 ] = type_l;
		rf_payload_receive[ debug_payload_length + 2 ] = 0;				//data length is zero
		debug_tuple_length_position = debug_payload_length + 2;
		debug_payload_length += 3;

		return 0;
	}
}


uint8_t DebugViaUsbridgeAddData( uint8_t datum )
{
	if (( RF_PAYLOAD_SIZE - debug_payload_length ) < 1 )
	{
		return 1;													// not enough space
	}
	else
	{
		rf_payload_receive[ debug_tuple_length_position ]++;					//count the acl tuple len up
		rf_payload_receive[ debug_payload_length ] = datum;					//place the data
		debug_payload_length += 1;
		return 0;
	}
}


#endif
