#ifndef UTIL_TIMEOUT_C
#define UTIL_TIMEOUT_C

#include <util/timeout.h>

xdata timeout_t timeout[MAX_TIMEOUTS];

void TimeoutUpdate( void )
{
	static xdata uint32_t last_time = RTCGetTime();
	xdata uint32_t current_time = RTCGetTime();
	xdata diff = 0;
	
	/* will be also correct if overflow occured since last update */
	diff = current_time - last_time;
		
	if( diff >= 1000UL )
	{ 
		uint8_t i;
		timeout_t d = (timeout_t) (diff / 1000UL);
		
		/* cycle through all timeouts and decrement according 
		 * to timespan since last update
		 */	 
		for( i = 0; i < MAX_TIMEOUTS; ++i )
		{
			if( timeout[i] > d )
				timeout[i] -= d;
			else
				timeout[i] = 0;
		}
		
		last_time = current_time - (diff % 1000UL);
	}		
}

timeout_t* TimeoutAllocate( void )
{
	/* declare as static so that it is *not* reset to 0 on each call */
	static uint8_t next_free_timeout = 0;
	
	/* check if still timeout available */
	if( next_free_timeout == MAX_TIMEOUTS )
		return NULL;
	
	/* initialise with 0 */
	timeout[next_free_timeout] = 0;

	/* return pointer to now allocated timeout
	 * set next_free_timeout to next allocatable timeout
	 */
	return &(timeout[next_free_timeout++]);
}


#endif /*UTIL_TIMEOUT_C*/
